@@ -69,8 +69,6 @@ void              ev_application_open_uri_at_dest    (EvApplication   *applicati
 						      EvLinkDest      *dest,
 						      EvWindowRunMode  mode,
 						      const gchar     *search_string,
-						      gboolean         unlink_temp_file,
-						      const gchar     *print_settings, 
 						      guint32          timestamp);
 void	          ev_application_open_uri_list       (EvApplication   *application,
 		  			              GSList          *uri_list,
@@ -81,9 +79,11 @@ GObject		 *ev_application_get_media_keys	     (EvApplication   *application);
 
 EggToolbarsModel *ev_application_get_toolbars_model  (EvApplication   *application);
 void              ev_application_save_toolbars_model (EvApplication   *application);
-void 		  ev_application_set_chooser_uri     (EvApplication   *application, 
+void 		  ev_application_set_filechooser_uri (EvApplication   *application,
+						      GtkFileChooserAction action,
 						      const gchar     *uri);
-const gchar	 *ev_application_get_chooser_uri     (EvApplication   *application);
+const gchar	 *ev_application_get_filechooser_uri (EvApplication   *application,
+						      GtkFileChooserAction action);
 void		  ev_application_screensaver_enable  (EvApplication   *application);
 void		  ev_application_screensaver_disable (EvApplication   *application);
 GtkPrintSettings *ev_application_get_print_settings  (EvApplication   *application);
@@ -93,6 +93,7 @@ GtkPageSetup     *ev_application_get_page_setup      (EvApplication   *applicati
 void              ev_application_set_page_setup      (EvApplication   *application,
 						      GtkPageSetup    *page_setup);
 const gchar      *ev_application_get_dot_dir         (EvApplication   *application);
+const gchar      *ev_application_get_data_dir         (EvApplication   *application);
 
 G_END_DECLS
 
