@@ -25,8 +25,7 @@
 
 #include <string.h>
 #include <math.h>
-
-#define PRINT_CONFIG_FILENAME	"ev-print-config.xml"
+#include <glib/gi18n.h>
 
 typedef struct
 {
@@ -212,98 +211,6 @@ ev_print_region_contents (GdkRegion *region)
 	g_free (rectangles);
 }
 
-#ifdef WITH_GNOME_PRINT
-gboolean
-using_pdf_printer (GnomePrintConfig *config)
-{
-	const guchar *driver;
-
-	driver = gnome_print_config_get (
-		config, (const guchar *)"Settings.Engine.Backend.Driver");
-
-	if (driver) {
-		if (!strcmp ((const gchar *)driver, "gnome-print-pdf"))
-			return TRUE;
-		else
-			return FALSE;
-	}
-
-	return FALSE;
-}
-
-gboolean
-using_postscript_printer (GnomePrintConfig *config)
-{
-	const guchar *driver;
-	const guchar *transport;
-
-	driver = gnome_print_config_get (
-		config, (const guchar *)"Settings.Engine.Backend.Driver");
-
-	transport = gnome_print_config_get (
-		config, (const guchar *)"Settings.Transport.Backend");
-
-	if (driver) {
-		if (!strcmp ((const gchar *)driver, "gnome-print-ps"))
-			return TRUE;
-		else
-			return FALSE;
-	} else 	if (transport) { /* these transports default to PostScript */
-		if (!strcmp ((const gchar *)transport, "CUPS"))
-			return TRUE;
-		else if (!strcmp ((const gchar *)transport, "LPD"))
-			return TRUE;
-		else if (!strcmp ((const gchar *)transport, "PAPI"))
-			return TRUE;
-	}
-
-	return FALSE;
-}
-
-GnomePrintConfig *
-load_print_config_from_file (void)
-{
-	GnomePrintConfig *print_config = NULL;
-	char *file_name, *contents = NULL;
-
-	file_name = g_build_filename (ev_dot_dir (), PRINT_CONFIG_FILENAME,
-				      NULL);
-
-	if (g_file_get_contents (file_name, &contents, NULL, NULL)) {
-		print_config = gnome_print_config_from_string (contents, 0);
-		g_free (contents);
-	}
-
-	if (print_config == NULL) {
-		print_config = gnome_print_config_default ();
-	}
-
-	g_free (file_name);
-
-	return print_config;
-}
-
-void
-save_print_config_to_file (GnomePrintConfig *config)
-{
-	char *file_name, *str;
-
-	g_return_if_fail (config != NULL);
-
-	str = gnome_print_config_to_string (config, 0);
-	if (str == NULL) return;
-
-	file_name = g_build_filename (ev_dot_dir (),
-				      PRINT_CONFIG_FILENAME,
-				      NULL);
-
-	g_file_set_contents (file_name, str, -1, NULL);
-
-	g_free (file_name);
-	g_free (str);
-}
-#endif /* WITH_GNOME_PRINT */
-
 static void
 ev_gui_sanitise_popup_position (GtkMenu *menu,
 				GtkWidget *widget,
@@ -410,3 +317,126 @@ get_screen_dpi (GtkWindow *window)
 	return (xdpi + ydpi) / 2.0;
 }
 
+
+void           
+file_chooser_dialog_add_writable_pixbuf_formats (GtkFileChooser *chooser)
+{
+	GSList *pixbuf_formats = NULL;
+	GSList *iter;
+	GtkFileFilter *filter;
+	int i;
+  
+	filter = gtk_file_filter_new();
+	gtk_file_filter_set_name (filter, _("By extension"));
+	g_object_set_data (G_OBJECT(filter), "pixbuf-format", NULL);
+	gtk_file_chooser_add_filter (chooser, filter);
+
+	pixbuf_formats = gdk_pixbuf_get_formats ();
+
+	for (iter = pixbuf_formats; iter; iter = iter->next) {
+		GdkPixbufFormat *format = iter->data;
+
+	        gchar *description, *name, *extensions;
+		gchar **extension_list, **mime_types;
+
+		if (gdk_pixbuf_format_is_disabled (format) ||
+	    	    !gdk_pixbuf_format_is_writable (format))
+		            continue;
+
+	        name = gdk_pixbuf_format_get_description (format);
+	        extension_list = gdk_pixbuf_format_get_extensions (format);
+	        extensions = g_strjoinv (", ", extension_list);
+		g_strfreev (extension_list);
+		description = g_strdup_printf ("%s (%s)", name, extensions);
+
+		filter = gtk_file_filter_new ();
+		gtk_file_filter_set_name (filter, description);
+		g_object_set_data (G_OBJECT (filter), "pixbuf-format", format);
+		gtk_file_chooser_add_filter (chooser, filter);
+
+		g_free (description);
+		g_free (extensions);
+		g_free (name);
+
+		mime_types = gdk_pixbuf_format_get_mime_types (format);
+		for (i = 0; mime_types[i] != 0; i++)
+			gtk_file_filter_add_mime_type (filter, mime_types[i]);
+		g_strfreev (mime_types);
+	}
+
+	g_slist_free (pixbuf_formats);
+}
+
+GdkPixbufFormat*
+get_gdk_pixbuf_format_by_extension (gchar *uri)
+{
+	GSList *pixbuf_formats = NULL;
+	GSList *iter;
+	int i;
+
+	pixbuf_formats = gdk_pixbuf_get_formats ();
+
+	for (iter = pixbuf_formats; iter; iter = iter->next) {
+		gchar **extension_list;
+		GdkPixbufFormat *format = iter->data;
+		
+		if (gdk_pixbuf_format_is_disabled (format) ||
+	    	    !gdk_pixbuf_format_is_writable (format))
+		            continue;
+
+	        extension_list = gdk_pixbuf_format_get_extensions (format);
+
+		for (i = 0; extension_list[i] != 0; i++) {
+			if (g_str_has_suffix (uri, extension_list[i])) {
+			    	g_slist_free (pixbuf_formats);
+				g_strfreev (extension_list);
+				return format;
+			}
+		}
+		g_strfreev (extension_list);
+	}
+
+	g_slist_free (pixbuf_formats);
+	return NULL;
+}
+
+#define XDIGIT(c) ((c) <= '9' ? (c) - '0' : ((c) & 0x4F) - 'A' + 10)
+#define HEXCHAR(s) ((XDIGIT (s[1]) << 4) + XDIGIT (s[2]))
+
+static char *
+uri_decoded_copy (const char *part, int length)
+{
+	unsigned char *s, *d;
+	char *decoded = g_strndup (part, length);
+
+	s = d = (unsigned char *)decoded;
+	do {
+		if (*s == '%') {
+			if (!g_ascii_isxdigit (s[1]) ||
+			    !g_ascii_isxdigit (s[2])) {
+				g_free (decoded);
+				return NULL;
+			}
+			*d++ = HEXCHAR (s);
+			s += 2;
+		} else
+			*d++ = *s;
+	} while (*s++);
+
+	return decoded;
+}
+
+char* escape_uri_for_display (const char *uri)
+{
+	GFile *file;
+	char *disp;
+	char *filename;
+
+	file = g_file_new_for_uri (uri);
+	filename = g_file_get_parse_name (file);
+	disp = uri_decoded_copy (filename, strlen (filename));
+	g_free (filename);
+	g_object_unref (file);
+
+	return disp;
+}
